<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <title>Redirecting...</title>
    <meta http-equiv="refresh" content="0; URL={{url}}">
    <link rel="canonical" href="{{url}}">
  </head>
  <body>
      <p>Redirecting to... <a href="{{url}}">{{url}}</a>.</p>
  </body>
</html>
